"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel
from flexprice.utils import FieldMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class GetTaskResultRequestTypedDict(TypedDict):
    workflow_id: str
    r"""Workflow ID"""


class GetTaskResultRequest(BaseModel):
    workflow_id: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""Workflow ID"""
