"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .datatype import DataType
from .filteroperatortype import FilterOperatorType
from .value import Value, ValueTypedDict
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class FilterConditionTypedDict(TypedDict):
    data_type: NotRequired[DataType]
    field: NotRequired[str]
    operator: NotRequired[FilterOperatorType]
    value: NotRequired[ValueTypedDict]


class FilterCondition(BaseModel):
    data_type: Optional[DataType] = None

    field: Optional[str] = None

    operator: Optional[FilterOperatorType] = None

    value: Optional[Value] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data_type", "field", "operator", "value"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
