"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


PaymentMethodType = Union[
    Literal[
        "CARD",
        "ACH",
        "OFFLINE",
        "CREDITS",
        "PAYMENT_LINK",
    ],
    UnrecognizedStr,
]
