"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_ingesteventrequest import (
    DtoIngestEventRequest,
    DtoIngestEventRequestTypedDict,
)
from flexprice.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class DtoBulkIngestEventRequestTypedDict(TypedDict):
    events: List[DtoIngestEventRequestTypedDict]


class DtoBulkIngestEventRequest(BaseModel):
    events: List[DtoIngestEventRequest]
