"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_userresponse import DtoUserResponse, DtoUserResponseTypedDict
from .paginationresponse import PaginationResponse, PaginationResponseTypedDict
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoListUsersResponseTypedDict(TypedDict):
    items: NotRequired[List[DtoUserResponseTypedDict]]
    pagination: NotRequired[PaginationResponseTypedDict]


class DtoListUsersResponse(BaseModel):
    items: Optional[List[DtoUserResponse]] = None

    pagination: Optional[PaginationResponse] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["items", "pagination"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
