"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


CollectionMethod = Literal[
    "charge_automatically",
    "send_invoice",
]
