"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .entitlementusageresetperiod import EntitlementUsageResetPeriod
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoUpdateEntitlementRequestTypedDict(TypedDict):
    is_enabled: NotRequired[bool]
    is_soft_limit: NotRequired[bool]
    static_value: NotRequired[str]
    usage_limit: NotRequired[int]
    usage_reset_period: NotRequired[EntitlementUsageResetPeriod]


class DtoUpdateEntitlementRequest(BaseModel):
    is_enabled: Optional[bool] = None

    is_soft_limit: Optional[bool] = None

    static_value: Optional[str] = None

    usage_limit: Optional[int] = None

    usage_reset_period: Optional[EntitlementUsageResetPeriod] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "is_enabled",
                "is_soft_limit",
                "static_value",
                "usage_limit",
                "usage_reset_period",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
