"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ReportingUnitTypedDict(TypedDict):
    conversion_rate: NotRequired[float]
    r"""Multiplier: reporting_unit_value = unit_value * conversion_rate; must be > 0"""
    unit_plural: NotRequired[str]
    r"""Display unit label, plural (e.g. \"seconds\")"""
    unit_singular: NotRequired[str]
    r"""Display unit label, singular (e.g. \"second\")"""


class ReportingUnit(BaseModel):
    conversion_rate: Optional[float] = None
    r"""Multiplier: reporting_unit_value = unit_value * conversion_rate; must be > 0"""

    unit_plural: Optional[str] = None
    r"""Display unit label, plural (e.g. \"seconds\")"""

    unit_singular: Optional[str] = None
    r"""Display unit label, singular (e.g. \"second\")"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["conversion_rate", "unit_plural", "unit_singular"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
