"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice.types import UnrecognizedStr
from typing import Literal, Union


WindowSize = Union[
    Literal[
        "MONTH",
        "MINUTE",
        "15MIN",
        "30MIN",
        "HOUR",
        "3HOUR",
        "6HOUR",
        "12HOUR",
        "DAY",
        "WEEK",
    ],
    UnrecognizedStr,
]
